//
//  SCAdFacadeDelegate.h
//  SmartclipSDKiOS
//
//  Copyright © 2018 smartclip. All rights reserved.
//
#import <AVKit/AVKit.h>
#import <SmartclipSDKiOS/SCAdSlotController.h>
#import <SmartclipSDKiOS/SCPlayerEventListener.h>
#import <SmartclipSDKiOS/SCAdListener.h>

#ifndef SCAdFacadeDelegate_h
#define SCAdFacadeDelegate_h

@class SCAdEvent, SCElementSize, SCAdEnvironment;
/**
 * The SCAdFacadeDelegate protocol needs to be implemented by the
 * video player implementation
 * it receives its commands and delivers information through this protocol
 */
@protocol SCAdFacadeDelegate <NSObject>
@optional

/// if you use customizableClickThroughAlert, add this function
- (void)displayClickThroughAlert;
/// if you want to support subtitles (captions), implement this function in addition to the
/// normal loadAdWithUrlString function (see below)
- (BOOL)loadAdWithCaptions:(NSDictionary* _Nonnull)loadDict disableSeeking:(BOOL)disableSeeking completion:(nullable SCEmptyCompletionBlock)completion;
///this function is required for VMAP, but not needed for pure VAST
- (nullable SCAdEnvironment*)getEnvironmentVarsWith:(NSString * _Nonnull)adBreakType;

@required
/// The player should load the ad from this url and is told to disable/enable seeking
- (BOOL)loadAdWithUrlString:(NSString* _Nonnull)urlString disableSeeking:(BOOL)disableSeeking completion:(nullable SCEmptyCompletionBlock)completion;
/// The player should start its playback
- (void)startPlayback;
/// The player should pause its playback
- (void)pausePlayback;
/// The player should resume its playback
- (void)resumePlayback;
/// The player is being asked if it is paused
- (BOOL)isPaused;
/// The player should mute sound
- (void)mutePlayer;
/// The player should unmute sound
- (void)unmutePlayer;
/// The player is being asked if it is muted
- (BOOL)isMuted;
/// The player should seek from its current position to a time offset
- (void)seek:(NSInteger)offset;
/// The player must give back its current playback position
- (double)getCurrentTime;
/// The player must give back the duration of its current video
- (double)getDuration;
/// Not used at the moment
- (NSString*_Nullable)getVideoElement;
/// The player should set its volume to the desired value (unused at the moment)
- (void)setVolume:(CGFloat)volume;
/// The player should give back the current volume (currently unused)
- (CGFloat)getVolume;
/// The player should lock the current content source
- (void)lockContent;
/// Start content playback with this playerController
- (void)playContentVideoWith:(NSURL*_Nonnull)url;
/// The player should give back its current content source
- (NSString *_Nullable)getCurrentContentSource;
/// The player should release its source to the content source (after an adSlot is finished)
- (BOOL)releaseContent;
/// Callback function that is being called for every single event
- (void)onEventCallback:(SCAdEvent *_Nonnull)event;
/// Give back an error, if a playback error occured
- (nullable NSString*)getError;
/// If your player is connected to the sequencer it should ask the sequencer for the adInfo and give it back
- (void)getAdInfo:(SCAdInfoCompletionBlock _Nonnull )completion;
/// Click through happened, pass it on to the view controller
- (void)displayClickThroughAlert:(UIAlertController* _Nonnull)alert;
/// Visibility of the adView changed, so pause or resume the player
- (void)onVisibilityChanged:(BOOL)visible;
/// Skip an ad
- (void)skipAd;
/// If your player is connected to the sequencer it should ask the sequencer for the adError object, if an error was detected
- (void)getAdError:(SCAdErrorCompletionBlock _Nonnull )completion;
/// Give back the current player frame and viewMode (normal or fullscreen)
- (nullable SCElementSize*)getPlayerSize;
/// Give back the frame of the viewPort and the viewMode (normal or fullscreen)
- (nullable SCElementSize*)getViewportSize;
/// return AVPlayer
- (nonnull AVPlayer*)getAVPlayer;
/// set eventListener
- (void)setEventListener:(id<SCPlayerEventListener> _Nonnull)listener;
/// set playerListener
- (void)setPlayerListener:(id<SCAdListener> _Nonnull)listener;
/// cleanup
- (void)cleanup;
@end

#endif /* SCAdFacadeDelegate_h */
